/*
 * ifaddrs.h compatibility shim
 */

#ifdef HAVE_IFADDRS_H

#include_next <ifaddrs.h>

#else

#ifndef LIBCOMPAT_IFADDRS_H
#define LIBCOMPAT_IFADDRS_H

/* The following code was found on the OpenSolaris networking-discuss
 * mailing list (http://markmail.org/message/t74kvl63savhzrzp) in a post
 * by James Carlson.
 */

/*
 * Copyright (c) 2006 WIDE Project. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/types.h>

#undef ifa_broadaddr
#undef ifa_dstaddr
struct ifaddrs {
	struct ifaddrs	*ifa_next;	/* Pointer to next struct */
	char		*ifa_name;	/* Interface name */
	uint64_t	ifa_flags;	/* Interface flags */
	struct sockaddr	*ifa_addr;	/* Interface address */
	struct sockaddr	*ifa_netmask;	/* Interface netmask */
	struct sockaddr	*ifa_dstaddr;	/* P2P interface destination */
};
#define	ifa_broadaddr	ifa_dstaddr

extern int getifaddrs(struct ifaddrs **);
extern void freeifaddrs(struct ifaddrs *);

#endif

#endif
